from .base import (get_x_y,  # noqa: F401
                   load_arff_files_standardized,  # noqa: F401
                   load_aids,  # noqa: F401
                   load_breast_cancer,  # noqa: F401
                   load_flchain,  # noqa: F401
                   load_gbsg2,  # noqa: F401
                   load_veterans_lung_cancer,  # noqa: F401
                   load_whas500)  # noqa: F401



# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import os
import os.path
from distutils.version import LooseVersion

CYTHON_MIN_VERSION = '0.23'


def build_from_c_and_cpp_files(extensions):
    """Modify the extensions to build from the .c and .cpp files.
    This is useful for releases, this way cython is not required to
    run python setup.py install.
    """
    for extension in extensions:
        sources = []
        for sfile in extension.sources:
            path, ext = os.path.splitext(sfile)
            if ext in ('.pyx', '.py'):
                if extension.language == 'c++':
                    ext = '.cpp'
                else:
                    ext = '.c'
                sfile = path + ext
            sources.append(sfile)
        extension.sources = sources


def maybe_cythonize_extensions(top_path, config):
    """Tweaks for building extensions between release and development mode."""
    is_release = os.path.exists(os.path.join(top_path, 'PKG-INFO'))

    if is_release:
        build_from_c_and_cpp_files(config.ext_modules)
    else:
        message = ('Please install cython with a version >= {0} in order '
                   'to build a scikit-survival development version.').format(
                       CYTHON_MIN_VERSION)
        try:
            import Cython
            if LooseVersion(Cython.__version__) < CYTHON_MIN_VERSION:
                message += ' Your version of Cython was {0}.'.format(
                    Cython.__version__)
                raise ValueError(message)
            from Cython.Build import cythonize
        except ImportError as exc:
            exc.args += (message,)
            raise

        # http://docs.cython.org/en/latest/src/userguide/source_files_and_compilation.html#cythonize-arguments
        directives = {'language_level': '3'}
        cy_cov = os.environ.get('CYTHON_COVERAGE', False)
        if cy_cov:
            directives['linetrace'] = True
            macros = [('CYTHON_TRACE', '1'), ('CYTHON_TRACE_NOGIL', '1')]
        else:
            macros = []

        config.ext_modules = cythonize(
            config.ext_modules,
            compiler_directives=directives)

        for e in config.ext_modules:
            e.define_macros.extend(macros)


def configuration(parent_package='', top_path=None):
    from numpy.distutils.misc_util import Configuration

    config = Configuration('sksurv', parent_package, top_path)
    config.add_subpackage('bintrees')
    config.add_subpackage('datasets')
    config.add_subpackage('ensemble')
    config.add_subpackage('io')
    config.add_subpackage('kernels')
    config.add_subpackage('linear_model')
    config.add_subpackage('meta')
    config.add_subpackage('svm')

    maybe_cythonize_extensions(top_path, config)

    return config


if __name__ == '__main__':
    from numpy.distutils.core import setup
    setup(**configuration(top_path='').todict())



from pathlib import Path
import numpy


def configuration(parent_package='', top_path=None):
    from numpy.distutils.misc_util import Configuration

    config = Configuration('linear_model', parent_package, top_path)

    eigen_root = Path('src', 'eigen')
    eigen_src = Path(__file__).parent / eigen_root / 'Eigen'
    if not eigen_src.is_dir():
        raise RuntimeError("{} directory not found. You might have to run "
                           "'git submodule update --init'.".format(eigen_src))

    config.add_extension('_coxnet',
                         sources=['_coxnet.pyx'],
                         include_dirs=[numpy.get_include(), 'src',
                                       str(eigen_root)],
                         language='c++',
                         extra_compile_args=["-std=c++11"])

    return config


if __name__ == '__main__':
    from numpy.distutils.core import setup
    setup(**configuration(top_path='').todict())



import numpy
from numpy.testing import assert_array_almost_equal

from sksurv.linear_model import IPCRidge
from sksurv.testing import assert_cindex_almost_equal


class TestIPCRidge(object):

    @staticmethod
    def test_fit(make_whas500):
        whas500 = make_whas500()
        model = IPCRidge()
        model.fit(whas500.x, whas500.y)

        assert round(abs(model.intercept_ - 5.8673567124629571), 7) == 0
        expected = numpy.array([0.168517, -0.249717, 2.18515, 0.536795, -0.514571, 0.091203,
                                0.613006, 0.480385, -0.055949, 0.238529, -0.127148, -0.144134,
                                -1.625041, -0.217469])
        assert_array_almost_equal(model.coef_, expected)

    @staticmethod
    def test_predict(make_whas500):
        whas500 = make_whas500()
        model = IPCRidge()
        model.fit(whas500.x[:400], whas500.y[:400])

        x_test = whas500.x[400:]
        y_test = whas500.y[400:]
        p = model.predict(x_test)
        assert_cindex_almost_equal(y_test['fstat'], y_test['lenfol'], -p,
                                   (0.66925817946226107, 2066, 1021, 0, 1))

        assert model.score(x_test, y_test) == 1.0 - 0.66925817946226107



from collections import OrderedDict

import numpy
from numpy.testing import assert_array_equal
import pandas.util.testing as tm
import pandas
import pytest

from sksurv.util import safe_concat, Surv


class TestUtil(object):
    @staticmethod
    def test_concat_numeric():
        rnd = numpy.random.RandomState(14)
        a = pandas.Series(rnd.randn(100), name="col_A")
        b = pandas.Series(rnd.randn(100), name="col_B")

        expected_df = pandas.DataFrame.from_dict(OrderedDict(
            [(a.name, a), (b.name, b)]
        ))

        actual_df = safe_concat((a, b), axis=1)

        tm.assert_frame_equal(actual_df, expected_df)

    @staticmethod
    def test_concat_numeric_categorical():
        rnd = numpy.random.RandomState(14)
        a = pandas.Series(rnd.randn(100), name="col_A")
        b = pandas.Series(pandas.Categorical.from_codes(
            rnd.binomial(4, 0.6, 100), ["C1", "C2", "C3", "C4", "C5"]), name="col_B")

        expected_df = pandas.DataFrame.from_dict(OrderedDict(
            [(a.name, a), (b.name, b)]
        ))

        actual_df = safe_concat((a, b), axis=1)

        tm.assert_frame_equal(actual_df, expected_df)

    @staticmethod
    def test_concat_categorical():
        rnd = numpy.random.RandomState(14)
        a = pandas.DataFrame.from_dict(OrderedDict([
            ("col_A", pandas.Series(pandas.Categorical.from_codes(
                rnd.binomial(2, 0.6, 100), ["C1", "C2", "C3"]), name="col_A")),
            ("col_B", rnd.randn(100))]))
        b = pandas.DataFrame.from_dict(OrderedDict([
            ("col_A", pandas.Series(pandas.Categorical.from_codes(
                rnd.binomial(2, 0.2, 100), ["C1", "C2", "C3"]), name="col_A")),
            ("col_B", rnd.randn(100))]))

        expected_series = pandas.DataFrame.from_dict(OrderedDict([
            ("col_A", pandas.Series(pandas.Categorical.from_codes(
                numpy.concatenate((a.col_A.cat.codes.values, b.col_A.cat.codes.values)),
                ["C1", "C2", "C3"]
            ))),
            ("col_B", numpy.concatenate((a.col_B.values, b.col_B.values)))
        ]))
        expected_series.index = pandas.Index(a.index.tolist() + b.index.tolist())

        actual_series = safe_concat((a, b), axis=0)

        tm.assert_frame_equal(actual_series, expected_series)

    @staticmethod
    def test_concat_categorical_mismatch():
        rnd = numpy.random.RandomState(14)
        a = pandas.DataFrame.from_dict(OrderedDict([
            ("col_A", pandas.Series(pandas.Categorical.from_codes(
                rnd.binomial(2, 0.6, 100), ["C1", "C2", "C3"]), name="col_A")),
            ("col_B", rnd.randn(100))]))
        b = pandas.DataFrame.from_dict(OrderedDict([
            ("col_A", pandas.Series(pandas.Categorical.from_codes(
                rnd.binomial(3, 0.6, 100), ["C1", "C2", "C3", "C4"]), name="col_A")),
            ("col_B", rnd.randn(100))]))

        with pytest.raises(ValueError, match="categories for column col_A do not match"):
            safe_concat((a, b), axis=0)

    @staticmethod
    def test_concat_dataframe_numeric_categorical():
        rnd = numpy.random.RandomState(14)
        numeric_df = pandas.DataFrame.from_dict(OrderedDict(
            [("col_A", rnd.randn(100)), ("col_B", rnd.randn(100))]
        ))

        cat_series = pandas.Series(pandas.Categorical.from_codes(
            rnd.binomial(4, 0.6, 100), ["C1", "C2", "C3", "C4", "C5"]), name="col_C")

        expected_df = numeric_df.copy()
        expected_df["col_C"] = cat_series

        actual_df = safe_concat((numeric_df, cat_series), axis=1)

        tm.assert_frame_equal(actual_df, expected_df)

    @staticmethod
    def test_concat_duplicate_columns():
        rnd = numpy.random.RandomState(14)
        numeric_df = pandas.DataFrame.from_dict(OrderedDict([
            ("col_N", rnd.randn(100)), ("col_B", rnd.randn(100)),
            ("col_A", pandas.Series(pandas.Categorical.from_codes(
                rnd.binomial(4, 0.2, 100), ["C1", "C2", "C3", "C4", "C5"]), name="col_A")),
        ]))

        cat_df = pandas.DataFrame.from_dict(OrderedDict([
            ("col_A", pandas.Series(pandas.Categorical.from_codes(
                rnd.binomial(4, 0.6, 100), ["C1", "C2", "C3", "C4", "C5"]), name="col_A")),
            ("col_C", pandas.Series(pandas.Categorical.from_codes(
                rnd.binomial(1, 0.6, 100), ["Yes", "No"]), name="col_C")),
        ]))

        with pytest.raises(ValueError, match="duplicate columns col_A"):
            safe_concat((numeric_df, cat_df), axis=1)


@pytest.fixture
def surv_arrays():
    event = numpy.random.binomial(1, 0.5, size=100)
    time = numpy.exp(numpy.random.randn(100))
    return event, time


@pytest.fixture
def surv_data_frame():
    df = pandas.DataFrame({'event': numpy.random.binomial(1, 0.5, size=100),
                           'time': numpy.exp(numpy.random.randn(100))})
    return df


class TestSurv(object):

    @staticmethod
    def test_from_list(surv_arrays):
        event, time = surv_arrays

        expected = numpy.empty(dtype=[('event', bool), ('time', float)], shape=100)
        expected['event'] = event.astype(bool)
        expected['time'] = time

        y = Surv.from_arrays(list(event.astype(bool)), list(time))
        assert_array_equal(y, expected)

    @staticmethod
    def test_from_array_bool(surv_arrays):
        event, time = surv_arrays

        expected = numpy.empty(dtype=[('event', bool), ('time', float)], shape=100)
        expected['event'] = event.astype(bool)
        expected['time'] = time

        y = Surv.from_arrays(event.astype(bool), time)
        assert_array_equal(y, expected)

    @staticmethod
    def test_from_array_with_names(surv_arrays):
        event, time = surv_arrays

        expected = numpy.empty(dtype=[('death', bool), ('survival_time', float)], shape=100)
        expected['death'] = event.astype(bool)
        expected['survival_time'] = time

        y = Surv.from_arrays(event.astype(bool), time, name_time='survival_time', name_event='death')
        assert_array_equal(y, expected)

    @staticmethod
    def test_from_array_with_one_name_1(surv_arrays):
        event, time = surv_arrays

        expected = numpy.empty(dtype=[('death', bool), ('time', float)], shape=100)
        expected['death'] = event.astype(bool)
        expected['time'] = time

        y = Surv.from_arrays(event.astype(bool), time, name_event='death')
        assert_array_equal(y, expected)

    @staticmethod
    def test_from_array_with_one_name_2(surv_arrays):
        event, time = surv_arrays

        expected = numpy.empty(dtype=[('event', bool), ('survival_time', float)], shape=100)
        expected['event'] = event.astype(bool)
        expected['survival_time'] = time

        y = Surv.from_arrays(event.astype(bool), time, name_time='survival_time')
        assert_array_equal(y, expected)

    @staticmethod
    def test_from_array_int_event(surv_arrays):
        event, time = surv_arrays

        expected = numpy.empty(dtype=[('event', bool), ('time', float)], shape=100)
        expected['event'] = event.astype(bool)
        expected['time'] = time

        y = Surv.from_arrays(event, time)
        assert_array_equal(y, expected)

    @staticmethod
    def test_from_array_int_time(surv_arrays):
        event, time = surv_arrays
        time += 1
        time *= time

        expected = numpy.empty(dtype=[('event', bool), ('time', float)], shape=100)
        expected['event'] = event.astype(bool)
        expected['time'] = time.astype(int)

        y = Surv.from_arrays(event.astype(bool), time.astype(int))
        assert_array_equal(y, expected)

    @staticmethod
    def test_from_array_float(surv_arrays):
        event, time = surv_arrays

        expected = numpy.empty(dtype=[('event', bool), ('time', float)], shape=100)
        expected['event'] = event.astype(bool)
        expected['time'] = time

        y = Surv.from_arrays(event.astype(float), time)
        assert_array_equal(y, expected)

    @staticmethod
    def test_from_array_shape_mismatch(surv_arrays):
        event, time = surv_arrays

        msg = "Found input variables with inconsistent numbers of samples"
        with pytest.raises(ValueError, match=msg):
            Surv.from_arrays(event[1:], time)

        with pytest.raises(ValueError, match=msg):
            Surv.from_arrays(event, time[1:])

    @staticmethod
    def test_from_array_event_value_wrong_1(surv_arrays):
        event, time = surv_arrays
        event += 1

        with pytest.raises(ValueError,
                           match="non-boolean event indicator must contain 0 and 1 only"):
            Surv.from_arrays(event, time)

    @staticmethod
    def test_from_array_event_value_wrong_2(surv_arrays):
        event, time = surv_arrays
        event -= 1

        with pytest.raises(ValueError,
                           match="non-boolean event indicator must contain 0 and 1 only"):
            Surv.from_arrays(event, time)

    @staticmethod
    def test_from_array_event_value_wrong_3(surv_arrays):
        event, time = surv_arrays
        event[event == 0] = 3

        with pytest.raises(ValueError,
                           match="non-boolean event indicator must contain 0 and 1 only"):
            Surv.from_arrays(event, time)

    @staticmethod
    def test_from_array_event_value_wrong_4(surv_arrays):
        event, time = surv_arrays
        event[1] = 3

        with pytest.raises(ValueError,
                           match="event indicator must be binary"):
            Surv.from_arrays(event, time)

    @staticmethod
    def test_from_array_event_value_wrong_5(surv_arrays):
        event, time = surv_arrays
        event = numpy.arange(event.shape[0])

        with pytest.raises(ValueError,
                           match="event indicator must be binary"):
            Surv.from_arrays(event, time)

    @staticmethod
    def test_from_array_names_match(surv_arrays):
        event, time = surv_arrays

        with pytest.raises(ValueError,
                           match="name_time must be different from name_event"):
            Surv.from_arrays(event, time,
                             name_event='time_and_event', name_time='time_and_event')

    @staticmethod
    def test_from_dataframe_bool(surv_data_frame):
        data = surv_data_frame
        data['event'] = data['event'].astype(bool)

        expected = numpy.empty(dtype=[('event', bool), ('time', float)], shape=100)
        expected['event'] = data['event']
        expected['time'] = data['time']

        y = Surv.from_dataframe('event', 'time', data)
        assert_array_equal(y, expected)

    @staticmethod
    def test_from_dataframe_int(surv_data_frame):
        data = surv_data_frame

        expected = numpy.empty(dtype=[('event', bool), ('time', float)], shape=100)
        expected['event'] = data['event'].astype(bool)
        expected['time'] = data['time']

        y = Surv.from_dataframe('event', 'time', data)
        assert_array_equal(y, expected)

    @staticmethod
    def test_from_dataframe_float(surv_data_frame):
        data = surv_data_frame
        data['event'] = data['event'].astype(float)

        expected = numpy.empty(dtype=[('event', bool), ('time', float)], shape=100)
        expected['event'] = data['event'].astype(bool)
        expected['time'] = data['time']

        y = Surv.from_dataframe('event', 'time', data)
        assert_array_equal(y, expected)

    @staticmethod
    def test_from_dataframe_no_str_columns(surv_data_frame):
        data = surv_data_frame
        data['event'] = data['event'].astype(bool)

        expected = numpy.empty(dtype=[('0', bool), ('1', float)], shape=100)
        expected['0'] = data['event']
        expected['1'] = data['time']

        y = Surv.from_dataframe(0, 1, data.rename(columns={'event': 0, 'time': 1}))
        assert_array_equal(y, expected)

    @staticmethod
    def test_from_dataframe_column_names(surv_data_frame):
        data = surv_data_frame.rename(columns={'event': 'death', 'time': 'time_to_death'})
        data['death'] = data['death'].astype(bool)

        expected = numpy.empty(dtype=[('death', bool), ('time_to_death', float)], shape=100)
        expected['death'] = data['death']
        expected['time_to_death'] = data['time_to_death']

        y = Surv.from_dataframe('death', 'time_to_death', data)
        assert_array_equal(y, expected)

    @staticmethod
    def test_from_dataframe_no_such_column(surv_data_frame):
        data = surv_data_frame
        data['event'] = data['event'].astype(bool)

        expected = numpy.empty(dtype=[('event', bool), ('time', float)], shape=100)
        expected['event'] = data['event']
        expected['time'] = data['time']

        with pytest.raises(KeyError,
                           match=r'the label \[unknown\] is not in the \[columns\]'):
            Surv.from_dataframe('unknown', 'time', data)

        with pytest.raises(KeyError,
                           match=r'the label \[unknown\] is not in the \[columns\]'):
            Surv.from_dataframe('event', 'unknown', data)

    @staticmethod
    def test_from_dataframe_wrong_class(surv_data_frame):
        data = surv_data_frame

        with pytest.raises(TypeError,
                           match=r"exepected pandas.DataFrame, but got <class 'dict'>"):
            Surv.from_dataframe('event', 'time', data.to_dict())

        with pytest.raises(TypeError,
                           match=r"exepected pandas.DataFrame, but got <class 'numpy.ndarray'>"):
            Surv.from_dataframe('event', 'time', data.values)



import numpy
from numpy.testing import assert_array_almost_equal, assert_array_equal
import pytest

from sklearn.exceptions import ConvergenceWarning
from sklearn.preprocessing import scale

from sksurv.svm.minlip import MinlipSurvivalAnalysis, HingeLossSurvivalSVM
from sksurv.datasets import load_gbsg2
from sksurv.column import encode_categorical
from sksurv.svm._minlip import create_difference_matrix
from sksurv.testing import assert_cindex_almost_equal
from sksurv.util import Surv


@pytest.fixture
def toy_data():
    x = numpy.array([[1., 1.],
                     [10.2, 15.],
                     [20., 5.],
                     [40, 30],
                     [45, 21],
                     [50, 36]])

    rnd = numpy.random.RandomState(0)
    t = rnd.exponential(scale=8, size=x.shape[0])
    t.sort()
    y = Surv.from_arrays([True, True, False, True, False, False],
                         t,
                         name_event='status')
    return x, y


@pytest.fixture
def toy_test_data():
    x = numpy.array([[1., 1.],
                     [40, 30],
                     [50, 36]])
    rnd = numpy.random.RandomState(136)
    x += rnd.randn(*x.shape)
    return x


@pytest.fixture
def gbsg2():
    x, y = load_gbsg2()
    x = encode_categorical(x)
    return x.values, y


class TestDifferenceMatrix(object):

    @staticmethod
    def test_toy_create_difference_matrix_direct_neighbor_without_censoring(toy_data):
        _, y = toy_data
        status = numpy.ones(y.shape, dtype=bool)
        mat = create_difference_matrix(status.astype(numpy.uint8), y["time"], kind="next")

        expected = numpy.zeros((5, 6), dtype=numpy.int8)
        expected[0, 0] = -1
        expected[0, 1] = 1
        expected[1, 1] = -1
        expected[1, 2] = 1
        expected[2, 2] = -1
        expected[2, 3] = 1
        expected[3, 3] = -1
        expected[3, 4] = 1
        expected[4, 4] = -1
        expected[4, 5] = 1

        assert_array_equal(expected, mat.toarray())

        # should return first-order differences of y["time"]
        actual_diff = mat.dot(y["time"])
        expected_diff = (y["time"][1:] - y["time"][:-1])

        assert_array_almost_equal(expected_diff, actual_diff)

    @staticmethod
    def test_toy_create_difference_matrix_direct_neighbor_without_censoring_shuffled(toy_data):
        _, y = toy_data
        status = numpy.ones(y.shape, dtype=bool)
        order = [3, 2, 5, 0, 1, 4]  # = [ 20.  11.  70.   3.   6.  37.]
        time = y["time"][order]
        mat = create_difference_matrix(status.astype(numpy.uint8), time, kind="next")

        expected = numpy.zeros((5, 6), dtype=numpy.int8)
        expected[0, 3] = -1
        expected[0, 4] = 1
        expected[1, 4] = -1
        expected[1, 1] = 1
        expected[2, 1] = -1
        expected[2, 0] = 1
        expected[3, 0] = -1
        expected[3, 5] = 1
        expected[4, 5] = -1
        expected[4, 2] = 1

        assert_array_equal(expected, mat.toarray())

        # should return first-order differences of y["time"]
        actual_diff = mat.dot(time)
        expected_diff = (y["time"][1:] - y["time"][:-1])

        assert_array_almost_equal(expected_diff, actual_diff)

    @staticmethod
    def test_toy_create_difference_matrix_direct_neighbor_with_censoring(toy_data):
        _, y = toy_data
        mat = create_difference_matrix(y["status"].astype(numpy.uint8), y["time"], kind="next")

        expected = numpy.zeros((3, 6), dtype=numpy.int8)
        expected[0, 0] = -1
        expected[0, 1] = 1
        expected[1, 1] = -1
        expected[1, 2] = 1
        expected[2, 3] = -1
        expected[2, 4] = 1

        assert_array_equal(expected, mat.toarray())

        # should return first-order differences of y["time"]
        actual_diff = mat.dot(y["time"])
        comparable_pairs = numpy.array([True, True, False, True, False])
        expected_diff = (y["time"][1:] - y["time"][:-1])[comparable_pairs]

        assert_array_almost_equal(expected_diff, actual_diff)

    @staticmethod
    def test_toy_create_difference_matrix_nearest_neighbor(toy_data):
        _, y = toy_data
        status = numpy.repeat(True, len(y))
        mat = create_difference_matrix(status.astype(numpy.uint8), y["time"], kind="nearest")

        expected = numpy.zeros((5, 6), dtype=numpy.int8)
        expected[0, 0] = -1
        expected[0, 1] = 1
        expected[1, 1] = -1
        expected[1, 2] = 1
        expected[2, 2] = -1
        expected[2, 3] = 1
        expected[3, 3] = -1
        expected[3, 4] = 1
        expected[4, 4] = -1
        expected[4, 5] = 1

        assert_array_equal(expected, mat.toarray())

    @staticmethod
    def test_toy_create_difference_matrix_nearest_neighbor_censored(toy_data):
        _, y = toy_data
        mat = create_difference_matrix(y["status"].astype(numpy.uint8), y["time"], kind="nearest")

        expected = numpy.zeros((5, 6), dtype=numpy.int8)
        expected[0, 0] = -1
        expected[0, 1] = 1
        expected[1, 1] = -1
        expected[1, 2] = 1
        expected[2, 1] = -1
        expected[2, 3] = 1
        expected[3, 3] = -1
        expected[3, 4] = 1
        expected[4, 3] = -1
        expected[4, 5] = 1

        assert_array_equal(expected, mat.toarray())

    @staticmethod
    def test_toy_create_difference_matrix_full(toy_data):
        _, y = toy_data
        status = numpy.repeat(True, len(y))
        mat = create_difference_matrix(status.astype(numpy.uint8), y["time"], kind="all")

        expected = numpy.zeros((15, 6), dtype=numpy.int8)
        expected[0, 1] = 1
        expected[0, 0] = -1

        expected[1:3, 2] = 1
        expected[1, 0] = -1
        expected[2, 1] = -1

        expected[3:6, 3] = 1
        expected[3, 0] = -1
        expected[4, 1] = -1
        expected[5, 2] = -1

        expected[6:10, 4] = 1
        expected[6, 0] = -1
        expected[7, 1] = -1
        expected[8, 2] = -1
        expected[9, 3] = -1

        expected[10:15, 5] = 1
        expected[10, 0] = -1
        expected[11, 1] = -1
        expected[12, 2] = -1
        expected[13, 3] = -1
        expected[14, 4] = -1

        assert_array_equal(expected, mat.toarray())

    @staticmethod
    def test_toy_create_difference_matrix_full_censored(toy_data):
        _, y = toy_data
        mat = create_difference_matrix(y["status"].astype(numpy.uint8), y["time"], kind="all")

        expected = numpy.zeros((11, 6), dtype=numpy.int8)
        expected[0, 1] = 1
        expected[0, 0] = -1

        expected[1:3, 2] = 1
        expected[1, 0] = -1
        expected[2, 1] = -1

        expected[3:5, 3] = 1
        expected[3, 0] = -1
        expected[4, 1] = -1

        expected[5:8, 4] = 1
        expected[5, 0] = -1
        expected[6, 1] = -1
        expected[7, 3] = -1

        expected[8:12, 5] = 1
        expected[8, 0] = -1
        expected[9, 1] = -1
        expected[10, 3] = -1

        assert_array_equal(expected, mat.toarray())


class TestToyCvxpyExample(object):

    @property
    def minlip_model(self):
        return MinlipSurvivalAnalysis(solver="cvxpy", alpha=1, pairs="next")

    @property
    def svm_model(self):
        return HingeLossSurvivalSVM(solver="cvxpy", alpha=2.)

    def test_toy_minlip_fit_cvxpy(self, toy_data):
        x, y = toy_data
        m = self.minlip_model
        m.set_params(alpha=2)
        m.fit(x, y)

        assert (1, x.shape[0]) == m.coef_.shape
        assert 1 == m.coef0
        expected_coef = numpy.array([
            [-0.011728003147, 0.011728002895, 0.000000000252,
             -0.017524801335, 0.017524801335, 0.]])
        assert_array_almost_equal(m.coef_, expected_coef)

    def test_toy_minlip_timeit(self, toy_data):
        x, y = toy_data
        m = self.minlip_model
        m.set_params(timeit=7)
        m.fit(x, y)

        assert 7 == len(m.timings_)

    def test_toy_minlip_predict_1_cvxpy(self, toy_data):
        x, y = toy_data
        m = self.minlip_model
        m.fit(x, y)

        p = m.predict(x)
        assert_cindex_almost_equal(y['status'], y['time'], p,
                                   (1.0, 11, 0, 0, 0))

    def test_toy_minlip_predict_2_cvxpy(self, toy_data, toy_test_data):
        x, y = toy_data
        m = self.minlip_model
        m.set_params(pairs="next")
        y = y.copy()
        y["time"] = numpy.arange(1, 7)
        sd = numpy.std(x, axis=0)
        m.fit(x / sd, y)

        p = m.predict(toy_test_data / sd)
        expected = numpy.array([-0.033523879826, -1.878228488294, -2.410824233892])
        assert_array_almost_equal(expected, p, decimal=5)

    def test_toy_hinge_fit(self, toy_data):
        x, y = toy_data
        m = self.svm_model
        sd = numpy.std(x, axis=0)
        m.fit(x / sd, y)

        assert (1, x.shape[0]) == m.coef_.shape
        assert 1 == m.coef0
        expected_coef = numpy.array([
            [-1.893832101337, 1.083653895940, 0.810178205398,
             -2., 2., 0.]])
        assert_array_almost_equal(m.coef_, expected_coef)

        p = m.predict(x / sd)
        assert_cindex_almost_equal(y['status'], y['time'], p,
                                   (1.0, 11, 0, 0, 0))

    def test_toy_hinge_predict_cvxpy(self, toy_data, toy_test_data):
        x, y = toy_data
        m = self.svm_model
        sd = numpy.std(x, axis=0)
        m.fit(x / sd, y)

        p = m.predict(toy_test_data / sd)
        expected = numpy.array([-0.090550891252, -4.213744335308, -5.252123739017])
        assert_array_almost_equal(expected, p, decimal=5)

    def test_toy_hinge_nearest_fit(self, toy_data):
        x, y = toy_data
        m = self.svm_model
        m.set_params(pairs="nearest")
        sd = numpy.std(x, axis=0)
        m.fit(x / sd, y)

        assert(1, x.shape[0]) == m.coef_.shape
        assert 1 == m.coef0
        expected_coef = numpy.array([
            [-1.893832101337, 1.083653895940, 0.810178205398,
             -2., 2., 0.]])
        assert_array_almost_equal(m.coef_, expected_coef)

        p = m.predict(x / sd)
        assert_cindex_almost_equal(y['status'], y['time'], p,
                                   (1.0, 11, 0, 0, 0))

    def test_toy_hinge_nearest_predict_cvxpy(self, toy_data, toy_test_data):
        x, y = toy_data
        m = self.svm_model
        m.set_params(pairs="nearest")
        sd = numpy.std(x, axis=0)
        m.fit(x / sd, y)

        p = m.predict(toy_test_data / sd)
        expected = numpy.array([-0.090550891252, -4.213744335308, -5.252123739017])
        assert_array_almost_equal(expected, p, decimal=5)


def has_cvxopt():
    try:
        import cvxopt  # noqa: F401
    except ImportError:
        return False
    return True


@pytest.mark.skipif(not has_cvxopt(), reason='no cvxopt installed')
class TestToyCvxoptExample(object):

    @property
    def minlip_model(self):
        return MinlipSurvivalAnalysis(solver="cvxopt", alpha=1, pairs="next", max_iter=1000)

    @property
    def svm_model(self):
        return HingeLossSurvivalSVM(solver="cvxopt", alpha=1, max_iter=1000)

    def test_toy_minlip_fit_cvxopt(self, toy_data):
        x, y = toy_data
        m = self.minlip_model
        m.fit(x, y)

        assert (1, x.shape[0]) == m.coef_.shape
        assert 1 == m.coef0
        expected_coef = numpy.array([
            [-0.011727707619, 0.011727690349, 0.000000017270,
             -0.017525505057, 0.017525505057, 0.]])
        assert_array_almost_equal(m.coef_, expected_coef)

    def test_toy_minlip_predict_1_cvxopt(self, toy_data):
        x, y = toy_data
        m = self.minlip_model
        m.fit(x, y)

        p = m.predict(x)
        assert_cindex_almost_equal(y['status'], y['time'], p,
                                   (1.0, 11, 0, 0, 0))

    def test_toy_minlip_predict_2_cvxopt(self, toy_data):
        x, y = toy_data
        m = self.minlip_model
        y = y.copy()
        y["time"] = numpy.arange(1, 7)
        m.fit(x, y)

        p = m.predict(numpy.array([[3, 4], [41, 29]]))
        assert_array_almost_equal(numpy.array([-0.34162365, -5.37435297]), p)

    def test_toy_hinge_predict_cvxopt(self, toy_data):
        x, y = toy_data
        m = self.svm_model
        m.fit(x, y)

        p = m.predict(numpy.array([[3, 4], [41, 29]]))
        assert_array_almost_equal(numpy.array([-0.341622, -5.374336]), p)

    def test_toy_hinge_nearest_predict_cvxopt(self, toy_data):
        x, y = toy_data
        m = self.svm_model
        m.set_params(pairs="nearest")
        m.fit(x, y)

        p = m.predict(numpy.array([[3, 4], [41, 29]]))
        assert_array_almost_equal(numpy.array([-0.341623, -5.374339]), p)


class TestMinlipCvxpy(object):

    @staticmethod
    def test_breast_cancer_cvxpy(gbsg2):
        x, y = gbsg2
        x = scale(x)
        m = MinlipSurvivalAnalysis(solver="cvxpy", alpha=1, pairs="next")
        m.fit(x, y)

        assert (1, x.shape[0]) == m.coef_.shape

        p = m.predict(x)
        assert_cindex_almost_equal(y['cens'], y['time'], p,
                                   (0.5990741854033906, 79720, 53352, 0, 42))

    @staticmethod
    @pytest.mark.slow
    def test_breast_cancer_rbf_cvxpy(gbsg2):
        x, y = gbsg2
        x = scale(x)
        m = MinlipSurvivalAnalysis(solver="cvxpy", alpha=1, kernel="rbf",
                                   gamma=1./8, pairs="next", max_iter=1000)
        m.fit(x, y)

        assert (1, x.shape[0]) == m.coef_.shape

        p = m.predict(x)
        assert_cindex_almost_equal(y['cens'], y['time'], p,
                                   (0.6105867500300589, 81252, 51820, 0, 42))

    @staticmethod
    def test_unknown_solver(gbsg2):
        x, y = gbsg2
        m = MinlipSurvivalAnalysis(solver=None)
        with pytest.raises(ValueError, match="unknown solver: None"):
            m.fit(x, y)

        m.set_params(solver="i don't know")
        with pytest.raises(ValueError, match="unknown solver: i don't know"):
            m.fit(x, y)

        m.set_params(solver=[('why', 'are'), ('you', 'doing this')])
        with pytest.raises(ValueError,
                           match=r"unknown solver: \[\('why', 'are'\), \('you', 'doing this'\)\]"):
            m.fit(x, y)

    @staticmethod
    @pytest.mark.slow
    def test_kernel_precomputed(gbsg2):
        x, y = gbsg2
        from sklearn.metrics.pairwise import pairwise_kernels
        from sklearn.utils.metaestimators import _safe_split

        m = MinlipSurvivalAnalysis(kernel="precomputed", solver="cvxpy")
        K = pairwise_kernels(x, metric="rbf", gamma=1./32)

        train_idx = numpy.arange(50, x.shape[0])
        test_idx = numpy.arange(50)
        X_fit, y_fit = _safe_split(m, K, y, train_idx)
        X_test, y_test = _safe_split(m, K, y, test_idx, train_idx)

        m.fit(X_fit, y_fit)

        p = m.predict(X_test)
        assert_cindex_almost_equal(y_test['cens'], y_test['time'], p,
                                   (0.626514131897712, 457, 269, 17, 0))


@pytest.mark.skipif(not has_cvxopt(), reason='no cvxopt installed')
class TestMinlipCvxopt(object):

    @property
    def model(self):
        return MinlipSurvivalAnalysis(solver="cvxopt", alpha=1, pairs="next", max_iter=1000)

    def test_breast_cancer_cvxopt(self, gbsg2):
        x, y = gbsg2
        m = self.model
        m.fit(x, y)

        assert (1, x.shape[0]) == m.coef_.shape

        p = m.predict(x)
        assert_cindex_almost_equal(y['cens'], y['time'], p,
                                   (0.59570007214139709, 79271, 53801, 0, 42))

    def test_breast_cancer_rbf_cvxopt(self, gbsg2):
        x, y = gbsg2
        x = scale(x)
        m = self.model
        m.set_params(kernel="rbf", gamma=1./8)
        m.fit(x, y)

        assert (1, x.shape[0]) == m.coef_.shape

        p = m.predict(x)
        assert_cindex_almost_equal(y['cens'], y['time'], p,
                                   (0.6106092942166647, 81255, 51817, 0, 42))

    @staticmethod
    def test_max_iter(gbsg2):
        x, y = gbsg2
        x = scale(x)
        m = MinlipSurvivalAnalysis(solver="cvxopt", alpha=1, kernel="polynomial",
                                   degree=2, pairs="next", max_iter=5)

        with pytest.warns(ConvergenceWarning,
                          match=r"cvxopt solver did not converge: unknown \(duality gap = [.0-9]+\)"):
            m.fit(x, y)



import os
import os.path
import sys
from pkg_resources import parse_requirements as _parse_requirements

from distutils.command.sdist import sdist
from setuptools import find_packages


def parse_requirements(filename):
    with open(filename) as fin:
        parsed_requirements = _parse_requirements(
            fin)
        requirements = [str(ir) for ir in parsed_requirements]
    return requirements


with open('README.rst') as fp:
    long_description = fp.read()


requirements = parse_requirements('requirements/prod.txt')


def configuration(parent_package='', top_path=None):
    if os.path.exists('MANIFEST'):
        os.remove('MANIFEST')

    from numpy.distutils.misc_util import Configuration

    config = Configuration(None, parent_package, top_path)

    # Avoid non-useful msg:
    # "Ignoring attempt to set 'name' (from ... "
    config.set_options(ignore_setup_xxx_py=True,
                       assume_default_configuration=True,
                       delegate_options_to_subpackages=True,
                       quiet=True)

    config.add_subpackage('sksurv')

    return config


def setup_package():
    metadata = dict(
        name='scikit-survival',
        url='https://github.com/sebp/scikit-survival',
        author='Sebastian Pölsterl',
        author_email='sebp@k-d-w.org',
        description='Survival analysis built on top of scikit-learn',
        long_description=long_description,
        license="GPLv3+",
        packages=find_packages(),
        classifiers=['Development Status :: 4 - Beta',
                     'Intended Audience :: Science/Research',
                     'Intended Audience :: Developers',
                     'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
                     'Operating System :: POSIX',
                     'Operating System :: Unix',
                     'Programming Language :: C',
                     'Programming Language :: Python',
                     'Programming Language :: Python :: 3.5',
                     'Programming Language :: Python :: 3.6',
                     'Programming Language :: Python :: 3.7',
                     'Topic :: Software Development',
                     'Topic :: Scientific/Engineering',
                     ],
        zip_safe=False,
        include_package_data=True,
        use_scm_version=True,
        setup_requires=['setuptools_scm'],
        install_requires=requirements,
        cmdclass={'sdist': sdist},
    )

    if (len(sys.argv) >= 2
            and ('--help' in sys.argv[1:] or sys.argv[1]
                 in ('--help-commands', 'egg_info', '--version', 'clean'))):

        # For these actions, NumPy is not required.
        from setuptools import setup
    else:
        from numpy.distutils.core import setup

        metadata['configuration'] = configuration

    setup(**metadata)


if __name__ == "__main__":
    py_version = sys.version_info[:2]
    if py_version < (3, 5):
        raise RuntimeError('Python 3.5 or later is required')

    setup_package()
